﻿using AirplaneWithRadar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirplaneWithRadar
{
    internal class MapsCollection
    {
        readonly Dictionary<string, MapWithSetAirplaneGeneric<IDrawningObject, AbstractMap>> _mapStorages;
        public List<string> Keys => _mapStorages.Keys.ToList();
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        /// <summary>
        /// Разделитель для записи информации по элементу словаря в файл
        /// </summary>
        private readonly char separatorDict = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char separatorData = ';';
        public MapsCollection(int pictureWidth, int pictureHeight)
        {
            _mapStorages = new Dictionary<string, MapWithSetAirplaneGeneric<IDrawningObject, AbstractMap>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddMap(string name, AbstractMap map)
        {
            if (!_mapStorages.ContainsKey(name))
            {
                _mapStorages.Add(name, new MapWithSetAirplaneGeneric<IDrawningObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
            }
        }
        public void DelMap(string name)
        {
            if (_mapStorages.ContainsKey(name)) _mapStorages.Remove(name);
        }
        public MapWithSetAirplaneGeneric<IDrawningObject, AbstractMap> this[string ind]
        {
            get
            {
                if (_mapStorages.ContainsKey(ind))
                {
                    return _mapStorages[ind];
                }
                return null;
            }
        }
        /// <summary>
        /// Сохранение информации по локомотивам в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns></returns>
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter sw = new(filename))
            {
                sw.Write($"MapsCollection{Environment.NewLine}");
                foreach (var storage in _mapStorages)
                {
                    sw.Write($"{storage.Key}{separatorDict}{storage.Value.GetData(separatorDict, separatorData)}{Environment.NewLine}");
                }
            }
        }
        /// <summary>
        /// Загрузка информации по локомотивам в депо из файла
        /// </summary>
        /// <param name="filename"></param>
        /// <returns></returns>
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException("Файл не найден");
            }
            using (StreamReader sr = new(filename))
            {
                string str = sr.ReadLine();
                if (!str.Contains("MapsCollection"))
                {
                    //если нет такой записи, то это не те данные
                    throw new FileFormatException("Формат данных в файле не правильный");
                }
                _mapStorages.Clear();
                while ((str = sr.ReadLine()) != null)
                {
                    var elem = str.Split(separatorDict);
                    AbstractMap map = null;
                    switch (elem[1])
                    {
                        case "SimpleMap":
                            map = new SimpleMap();
                            break;
                        case "SkyMap":
                            map = new SkyMap();
                            break;
                        case "FoggySkyMap":
                            map = new FoggySkyMap();
                            break;
                    }
                    _mapStorages.Add(elem[0], new MapWithSetAirplaneGeneric<IDrawningObject, AbstractMap>(_pictureWidth, _pictureHeight, map));
                    _mapStorages[elem[0]].LoadData(elem[2].Split(separatorData, StringSplitOptions.RemoveEmptyEntries));
                }
            }
        }
    }
}
